% Example for the trace norm solver
% min_X  0.5 ||X - A||^2 + \lambda ||X||_tr

rand('state', 1);   randn('state', 1);  warning off;
m = 100;   n = 50;
A = randn(m, n);
lambda = 1e-2;

% First use singular value thresholding
[U, S, V] = svd(A);
s0 = diag(S);  
idx = s0 > lambda;
U = U(:, idx);  s = s0(idx) - lambda; V = V(:, idx);
sln_svt = U*diag(s)*V';
opt_obj = 0.5*norm(sln_svt - A, 'fro')^2 + lambda * sum(s);
fprintf('optimal obj by SVT: %g\n', opt_obj);


% Second use our boosting solver
opts = init_opts();
opts.init_rank = 1;
X0 = zeros(m, n);
evalf = [];   % no functor for performance evaluation at each iteration
[sln_boost, opt_obj_boost_solver, iter, msg] = ...
          solve_trace_reg(@(X)quad_obj(X, A), lambda, X0, evalf, opts);
        
% Print the objective value of the optimal solution found by boosting
[U, S, V] = svd(sln_boost);
s0 = diag(S);  
opt_obj_boost = 0.5*norm(sln_boost - A, 'fro')^2 + lambda * sum(s0);
fprintf('optimal obj by boosting: %g\n', opt_obj_boost);
